from __future__ import absolute_import
__author__ = 'chris'
import copy
import json
from collections import OrderedDict

from django import forms

from .scripts import DjanguiForm
from ..backend import utils
from ..models import ScriptParameter


class DjanguiFormFactory(object):
    djangui_forms = {}

    @staticmethod
    def get_field(param, initial=None):
        field = param.form_field
        choices = json.loads(param.choices)
        field_kwargs = {'label': param.script_param.title(),
                        'required': param.required,
                        'help_text': param.param_help,
                        }
        if choices:
            field = 'ChoiceField'
            base_choices = [(None, '----')] if not param.required else []
            field_kwargs['choices'] = base_choices+[(str(i), str(i).title()) for i in choices]
        if field == 'FileField':
            if param.is_output:
                field = 'CharField'
                initial = None
            else:
                if initial is not None:
                    initial = utils.get_storage_object(initial) if not hasattr(initial, 'path') else initial
                    field_kwargs['widget'] = forms.ClearableFileInput()
        if initial is not None:
            field_kwargs['initial'] = initial
        field = getattr(forms, field)
        return field(**field_kwargs)

    def get_group_forms(self, model=None, pk=None, initial=None):
        pk = int(pk) if pk is not None else pk
        if pk is not None and pk in self.djangui_forms:
            if 'groups' in self.djangui_forms[pk]:
                return copy.deepcopy(self.djangui_forms[pk]['groups'])
        params = ScriptParameter.objects.filter(script=model)
        # set a reference to the object type for POST methods to use
        script_id_field = forms.CharField(widget=forms.HiddenInput)
        group_map = {}
        for param in params:
            field = self.get_field(param, initial=initial.get(param.slug) if initial else None)
            group_id = -1 if param.required else param.parameter_group.pk
            group_name = 'Required' if param.required else param.parameter_group.group_name
            group = group_map.get(group_id, {
                'group': group_name,
                'fields': OrderedDict()
            })
            group['fields'][param.slug] = field
            group_map[group_id] = group
        # create individual forms for each group
        group_map = OrderedDict([(i, group_map[i]) for i in sorted(group_map.keys())])
        d = {'action': model.get_url()}
        d['groups'] = []
        pk = model.pk
        for group_index, group in enumerate(group_map.iteritems()):
            group_pk, group_info = group
            form = DjanguiForm()
            if group_index == 0:
                form.fields['djangui_type'] = script_id_field
                form.fields['djangui_type'].initial = pk
            for field_pk, field in group_info['fields'].iteritems():
                form.fields[field_pk] = field
            d['groups'].append({'group_name': group_info['group'], 'form': str(form)})
        try:
            self.djangui_forms[pk]['groups'] = d
        except KeyError:
            self.djangui_forms[pk] = {'groups': d}
        # if the master form doesn't exist, create it while we have the model
        if 'master' not in self.djangui_forms[pk]:
            self.get_master_form(model=model, pk=pk)
        return d

    def get_master_form(self, model=None, pk=None):
        pk = int(pk) if pk is not None else pk
        if pk is not None and pk in self.djangui_forms:
            if 'master' in self.djangui_forms[pk]:
                return copy.deepcopy(self.djangui_forms[pk]['master'])
        master_form = DjanguiForm()
        params = ScriptParameter.objects.filter(script=model)
        # set a reference to the object type for POST methods to use
        pk = model.pk
        script_id_field = forms.CharField(widget=forms.HiddenInput)
        master_form.fields['djangui_type'] = script_id_field
        master_form.fields['djangui_type'].initial = pk

        for param in params:
            field = self.get_field(param)
            master_form.fields[param.slug] = field
        try:
            self.djangui_forms[pk]['master'] = master_form
        except KeyError:
            self.djangui_forms[pk] = {'master': master_form}
        # create the group forms while we have the model
        if 'groups' not in self.djangui_forms[pk]:
            self.get_group_forms(model=model, pk=pk)
        return master_form

DJ_FORM_FACTORY = DjanguiFormFactory()


from __future__ import absolute_import

from boto.utils import parse_ts
from django.core.files.storage import get_storage_class
from storages.backends.s3boto import S3BotoStorage

from . import settings as djangui_settings

# From https://github.com/jezdez/django_compressor/issues/100

class CachedS3BotoStorage(S3BotoStorage):
    def __init__(self, *args, **kwargs):
        super(CachedS3BotoStorage, self).__init__(*args, **kwargs)
        self.local_storage = get_storage_class('django.core.files.storage.FileSystemStorage')()

    def _open(self, name, mode='rb'):
        original_file = super(CachedS3BotoStorage, self)._open(name, mode=mode)
        if name.endswith('.gz'):
            return original_file
        return original_file

    def modified_time(self, name):
        name = self._normalize_name(self._clean_name(name))
        entry = self.entries.get(name)
        if entry is None:
            entry = self.bucket.get_key(self._encode_name(name))
        # Parse the last_modified string to a local datetime object.
        return parse_ts(entry.last_modified)

    def path(self, name):
        return self.local_storage.path(name)


#!/usr/bin/env python

__author__ = 'chris'
import argparse
import sys
import itertools

parser = argparse.ArgumentParser(description="https://projecteuler.net/problem=1 -- Find the sum of all the multiples of 3 or 5 below a number.")
parser.add_argument('--below', help='The number to find the sum of multiples below.', type=int, default=1000)

def main():
    args = parser.parse_args()
    below = args.below
    below = 100000 if below > 100000 else below
    total = sum(set([i for i in itertools.chain(xrange(3, below, 3), xrange(5,below,5))]))
    sys.stdout.write('The sum of numbers below {} is {}'.format(below, total))

if __name__ == "__main__":
    sys.exit(main())



from django.test import TestCase

from . import factories


class ScriptTestCase(TestCase):

    def test_script_creation(self):
        script = factories.TranslateScriptFactory()


class ScriptGroupTestCase(TestCase):

    def test_script_group_creation(self):
        group = factories.ScriptGroupFactory()


import os

BASE_DIR = os.path.split(__file__)[0]
DJANGUI_TEST_SCRIPTS = os.path.join(BASE_DIR, 'scripts')

SCRIPT_DATA = {
    'translate':
        {
            'data': {
                'djangui_type': '1',
                'job_name': 'abc',
                'sequence': 'ATATATATATA',
                'frame': '+3',
                'out': 'abc'
            }
        }
}


#!/usr/bin/env python
__author__ = 'chris'
description = """
Create a Django app with Djangui setup.
"""
import sys
import traceback
import os
env = os.environ
import imp
import subprocess
import shutil
import tempfile
from itertools import chain
from argparse import ArgumentParser
from django.template import Context, Engine


def main():
    parser = ArgumentParser(description=description)
    parser.add_argument('-p', '--project', help='The name of the django project to create.', type=str, required=True)
    args = parser.parse_args()

    project_name = args.project
    new_project = not os.path.exists(project_name)
    if not new_project:
        sys.stderr.write('Project {0} already exists.\n'.format(project_name))
        return 1
    env['DJANGO_SETTINGS_MODULE'] = ''
    subprocess.call(['django-admin', 'startproject', project_name], env=env)
    project_root = project_name
    project_base_dir = os.path.join(os.path.realpath(os.path.curdir), project_root, project_name)

    # search down until we find the conf folder
    djanguify_folder, folder_name = os.path.split(os.path.split(os.path.realpath(__file__))[0])
    while 'djangui' not in folder_name:
        djanguify_folder, folder_name = os.path.split(djanguify_folder)
    # at this point we should be at the base
    project_template_dir = os.path.join(djanguify_folder, folder_name, 'conf', 'project_template')


    context = Context(
        dict({
            'project_name': project_name,
        },
        autoescape=False
    ))

    template_files = []
    def walk_dir(templates, dest, filter=None):
        l = []
        for root, folders, files in os.walk(templates):
            for filename in files:
                if filename.endswith('.pyc') or (filter and filename not in filter):
                    continue
                relative_dir = '.{0}'.format(os.path.split(os.path.join(root, filename).replace(templates, ''))[0])
                l.append((os.path.join(root, filename), os.path.join(dest, relative_dir)))
        return l

    template_files += walk_dir(project_template_dir, project_base_dir)

    for template_file, dest_dir in template_files:
        template_file = open(template_file)
        content = template_file.read()
        content = content.decode('utf-8')
        template = Engine().from_string(content)
        content = template.render(context)
        content = content.encode('utf-8')
        to_name = os.path.join(dest_dir, os.path.split(template_file.name)[1])
        try:
            os.mkdir(dest_dir)
        except:
            pass
        with open(to_name, 'wb') as new_file:
            new_file.write(content)

    # move the django settings to the settings path so we don't have to chase Django changes.
    shutil.move(os.path.join(project_base_dir, 'settings.py'), os.path.join(project_base_dir, 'settings', 'django_settings.py'))
    # do the same with urls
    shutil.move(os.path.join(project_base_dir, 'urls.py'), os.path.join(project_base_dir, 'urls', 'django_urls.py'))
    env['DJANGO_SETTINGS_MODULE'] = '.'.join([project_name, 'settings', 'user_settings'])
    subprocess.call(['python', os.path.join(project_root, 'manage.py'), 'makemigrations'], env=env)
    subprocess.call(['python', os.path.join(project_root, 'manage.py'), 'migrate'], env=env)
    subprocess.call(['python', os.path.join(project_root, 'manage.py'), 'collectstatic', '--noinput'], env=env)
    sys.stdout.write("Please enter the project directory {0}, and run python manage.py createsuperuser and"
                     " python manage.py runserver to start. The admin can be found at localhost:8000/admin. You may also want to set your "
                     "DJANGO_SETTINGS_MODULE environment variable to {0}.settings \n".format(project_name))

if __name__ == "__main__":
    sys.exit(main())


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangui', '0004_djanguifile'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='djanguijob',
            name='celery_state',
        ),
        migrations.AlterField(
            model_name='djanguijob',
            name='status',
            field=models.CharField(default=b'submitted', max_length=255, choices=[(b'submitted', 'Submitted'), (b'running', 'Running'), (b'completed', 'Completed'), (b'deleted', 'Deleted')]),
        ),
    ]



#!/usr/bin/env python

__author__ = 'chris'
import argparse
import sys

parser = argparse.ArgumentParser(description="https://projecteuler.net/problem=6 -- Find the difference between the sum of the squares of the first one hundred natural numbers and the square of the sum.")
parser.add_argument('--cutoff', help='The number to find the sum of squares below.', type=int, default=1)

def main():
    args = parser.parse_args()
    cutoff = args.cutoff
    if cutoff < 0:
        sys.stderr.write('Numbers must be positive.')
        return -1
    cutoff = 1000 if cutoff > 1000 else cutoff+1
    answer = sum(xrange(cutoff))**2-sum([i**2 for i in xrange(cutoff)])
    sys.stdout.write('The difference of the sum of squares and the square of sums below {} is {}'.format(cutoff-1, answer))

if __name__ == "__main__":
    sys.exit(main())



from __future__ import absolute_import
__author__ = 'chris'
import json
import errno
import os
import sys
import traceback
from operator import itemgetter
from collections import OrderedDict

from django.conf import settings
from django.db import transaction
from django.db.utils import OperationalError
from django.core.files.storage import default_storage
from django.core.files import File
from django.utils.translation import gettext_lazy as _
from celery.contrib import rdb

from .argparse_specs import ArgParseNodeBuilder

from .. import settings as djangui_settings

def sanitize_name(name):
    return name.replace(' ', '_').replace('-', '_')


def sanitize_string(value):
    return value.replace('"', '\\"')

def get_storage(local=True):
    if djangui_settings.DJANGUI_EPHEMERAL_FILES:
        storage = default_storage.local_storage if local else default_storage
    else:
        storage = default_storage
    return storage

def get_job_commands(job=None):
    script = job.script
    com = ['python', script.get_script_path()]
    from ..models import ScriptParameters
    parameters = ScriptParameters.objects.filter(job=job)
    for param in parameters:
        com.extend(param.get_subprocess_value())
    return com

@transaction.atomic
def create_djangui_job(data):
    from ..models import Script, DjanguiJob, ScriptParameter, ScriptParameters
    script = Script.objects.get(pk=data.get('djangui_type'))
    job = DjanguiJob(user=data.get('user'), job_name=data.get('job_name'), job_description=data.get('job_description'),
                     script=script)
    job.save()
    parameters = {i.slug: i for i in ScriptParameter.objects.filter(slug__in=data.keys())}
    for i, v in data.iteritems():
        param = parameters.get(i)
        if param is not None:
            new_param = ScriptParameters(job=job, parameter=param)
            new_param.value = v
            new_param.save()
    return job


def get_master_form(model=None, pk=None):
    from ..forms.factory import DJ_FORM_FACTORY
    return DJ_FORM_FACTORY.get_master_form(model=model, pk=pk)


def get_form_groups(model=None, pk=None, initial=None):
    from ..forms.factory import DJ_FORM_FACTORY
    return DJ_FORM_FACTORY.get_group_forms(model=model, pk=pk, initial=initial)

def validate_form(form=None, data=None, files=None):
    form.add_djangui_fields()
    form.data = data if data is not None else {}
    form.files = files if files is not None else {}
    form.is_bound = True
    form.full_clean()

def load_scripts():
    from ..models import Script
    # select all the scripts we have, then divide them into groups
    dj_scripts = {}
    try:
        scripts = Script.objects.count()
    except OperationalError:
        # database not initialized yet
        return
    if scripts:
        scripts = Script.objects.all()
        for script in scripts:
            key = (script.script_group.group_order, script.script_group.pk)
            group = dj_scripts.get(key, {
                # 'url': reverse_lazy('script_group', kwargs={'script_group', script.script_group.slug}),
                'group': script.script_group, 'scripts': []
            })
            dj_scripts[key] = group
            try:
                # make sure we can load the form
                get_master_form(script)
                # the url mapping is script_group/script_name
                group['scripts'].append((script.script_order, script))
            except:
                sys.stdout.write('Traceback while loading {0}:\n {1}\n'.format(script, traceback.format_exc()))
                continue
    for group_pk, group_info in dj_scripts.iteritems():
        # order scripts
        group_info['scripts'].sort(key=itemgetter(0))
        group_info['scripts'] = [i[1] for i in group_info['scripts']]
    # order groups
    ordered_scripts = OrderedDict()
    for key in sorted(dj_scripts.keys(), key=itemgetter(0)):
        ordered_scripts[key[1]] = dj_scripts[key]
    settings.DJANGUI_SCRIPTS = ordered_scripts


def get_storage_object(path, local=False):
    storage = get_storage(local=local)
    obj = storage.open(path)
    obj.url = storage.url(path)
    obj.path = storage.path(path)
    return obj

def add_djangui_script(script=None, group=None):
    from djangui.models import Script, ScriptGroup, ScriptParameter, ScriptParameterGroup
    # if we have a script, it will at this point be saved in the model pointing to our file system, which may be
    # ephemeral. So the path attribute may not be implemented
    try:
        script_path = script.script_path.path
    except NotImplementedError:
        script_path = script.script_path.name

    script_obj, script = (script, get_storage_object(script_path, local=True).path) if isinstance(script, Script) else (False, script)
    if isinstance(group, ScriptGroup):
        group = group.group_name
    if group is None:
        group = 'Djangui Scripts'
    basename, extension = os.path.splitext(script)
    filename = os.path.split(basename)[1]

    parser = ArgParseNodeBuilder(script_name=filename, script_path=script)
    if not parser.valid:
        return (False, parser.error)
    # make our script
    d = parser.get_script_description()
    script_group, created = ScriptGroup.objects.get_or_create(group_name=group)
    if script_obj is False:
        djangui_script, created = Script.objects.get_or_create(script_group=script_group, script_description=d['description'],
                                                               script_path=script, script_name=d['name'])
    else:
        created = False
        if not script_obj.script_description:
            script_obj.script_description = d['description']
        if not script_obj.script_name:
            script_obj.script_name = d['name']
        # probably a much better way to avoid this recursion
        script_obj._add_script = False
        script_obj.save()
    if not created:
        if script_obj is False:
            djangui_script.script_version += 1
            djangui_script.save()
    if script_obj:
        djangui_script = script_obj
    # make our parameters
    CHOICE_MAPPING = {

    }
    for param_group_info in d['inputs']:
        param_group, created = ScriptParameterGroup.objects.get_or_create(group_name=param_group_info.get('group'), script=djangui_script)
        for param in param_group_info.get('nodes'):
            # TODO: fix choice limits
            #choice_limit = CHOICE_MAPPING[param.get('choice_limit')]
            # TODO: fix 'file' to be global in argparse
            is_out = True if param.get('upload', None) is False and param.get('type') == 'file' else not param.get('upload', False)
            script_param, created = ScriptParameter.objects.get_or_create(script=djangui_script, short_param=param['param'], script_param=param['name'],
                                                                          is_output=is_out, required=param.get('required', False),
                                                                          form_field=param['model'], default=param.get('default'), input_type=param.get('type'),
                                                                          choices=json.dumps(param.get('choices')), choice_limit=None,
                                                                          param_help=param.get('help'), is_checked=param.get('checked', False),
                                                                          parameter_group=param_group)
    # update our loaded scripts
    load_scripts()
    return (True, '')

def valid_user(obj, user):
    groups = obj.user_groups.all()
    from ..models import Script
    ret = {'valid': False, 'error': '', 'display': ''}
    if isinstance(obj, Script):
        from itertools import chain
        groups = list(chain(groups, obj.script_group.user_groups.all()))
    if not user.is_authenticated() and djangui_settings.DJANGUI_ALLOW_ANONYMOUS and len(groups) == 0:
        ret['valid'] = True
    elif groups:
        ret['error'] = _('You are not permitted to use this script')
    if not groups and obj.is_active:
        ret['valid'] = True
    if obj.is_active is True:
        if set(list(user.groups.all())) & set(list(groups)):
            ret['valid'] = True
    ret['display'] = 'disabled' if djangui_settings.DJANGUI_SHOW_LOCKED_SCRIPTS else 'hide'
    return ret

def mkdirs(path):
    try:
        os.makedirs(path)
    except OSError as exc:
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise

@transaction.atomic
def create_job_fileinfo(job):
    parameters = job.get_parameters()
    from ..models import DjanguiFile
    # first, create a reference to things the script explicitly created that is a parameter
    files = []
    for field in parameters:
        try:
            if field.parameter.form_field == 'FileField':
                value = field.value
                if value is None:
                    continue
                if isinstance(value, basestring):
                    # check if this was ever created and make a fileobject if so
                    if get_storage(local=True).exists(value):
                        if not get_storage(local=False).exists(value):
                            get_storage(local=False).save(value, File(get_storage(local=True).open(value)))
                        value = field.value
                    else:
                        field.force_value(None)
                        field.save()
                        continue
                d = {'parameter': field, 'file': value}
                files.append(d)
        except ValueError:
            continue


    known_files = {i['file'].name for i in files}
    # add the user_output files, these are things which may be missed by the model fields because the script
    # generated them without an explicit arguments reference in the script
    file_groups = {'archives': []}
    absbase = os.path.join(settings.MEDIA_ROOT, job.save_path)
    for filename in os.listdir(absbase):
        new_name = os.path.join(job.save_path, filename)
        if any([i.endswith(new_name) for i in known_files]):
            continue
        try:
            filepath = os.path.join(absbase, filename)
            if os.path.isdir(filepath):
                continue
            d = {'name': filename, 'file': get_storage_object(os.path.join(job.save_path, filename))}
            if filename.endswith('.tar.gz') or filename.endswith('.zip'):
                file_groups['archives'].append(d)
            else:
                files.append(d)
        except IOError:
            print absbase, filename, new_name
            print traceback.format_exc()
            continue


    # establish grouping by inferring common things
    file_groups['all'] = files
    import imghdr
    file_groups['images'] = []
    for filemodel in files:
        if imghdr.what(filemodel['file'].path):
            file_groups['images'].append(filemodel)
    file_groups['tabular'] = []
    file_groups['fasta'] = []

    def test_delimited(filepath):
        import csv
        with open(filepath, 'rb') as csv_file:
            try:
                dialect = csv.Sniffer().sniff(csv_file.read(1024*16), delimiters=',\t')
            except Exception as e:
                return False, None
            csv_file.seek(0)
            reader = csv.reader(csv_file, dialect)
            rows = []
            try:
                for index, entry in enumerate(reader):
                    if index == 5:
                        break
                    rows.append(entry)
            except Exception as e:
                return False, None
            return True, rows

    def test_fastx(filepath):
        # if we can be delimited by + or > we're maybe a fasta/q
        with open(filepath, 'rb') as fastx_file:
            sequences = OrderedDict()
            seq = ''
            header = ''
            for row_index, row in enumerate(fastx_file, 1):
                if row_index > 30:
                    break
                if row and row[0] == '>':
                    if seq:
                        sequences[header] = seq
                        seq = ''
                    header = row
                elif row:
                    # we bundle the fastq stuff in here since it's just a visual
                    seq += row
                if seq and header:
                    sequences[header] = seq
            if sequences:
                rows = []
                [rows.extend([i, v]) for i,v in sequences.iteritems()]
                return True, rows
        return False, None

    for filemodel in files:
        is_delimited, first_rows = test_delimited(filemodel['file'].path)
        if is_delimited:
            file_groups['tabular'].append(dict(filemodel, **{'preview': first_rows}))
        else:
            is_fasta, first_rows = test_fastx(filemodel['file'].path)
            if is_fasta:
                file_groups['fasta'].append(dict(filemodel, **{'preview': first_rows}))

    # Create our DjanguiFile models

    # mark things that are in groups so we don't add this to the 'all' category too to reduce redundancy
    grouped = set([i['file'].path for file_type, groups in file_groups.iteritems() for i in groups if file_type != 'all'])
    for file_type, group_files in file_groups.iteritems():
        for group_file in group_files:
            if file_type == 'all' and group_file['file'].path in grouped:
                continue
            try:
                dj_file = DjanguiFile(job=job, filetype=file_type, filepreview=json.dumps(group_file.get('preview')),
                                    parameter=group_file.get('parameter'))
                filepath = group_file['file'].path
                # make it relative to the root
                dj_file.filepath.name = filepath[filepath.find(settings.MEDIA_ROOT)+len(settings.MEDIA_ROOT)+1:]
                dj_file.save()
            except:
                sys.stderr.write('Error in saving DJFile: {}\n'.format(traceback.format_exc()))
                continue


def get_file_previews(job):
    from ..models import DjanguiFile
    files = DjanguiFile.objects.filter(job=job)
    groups = {'all': []}
    for file_info in files:
        filedict = {'name': file_info.filepath.name, 'preview': json.loads(file_info.filepreview),
                    'url': get_storage(local=False).url(file_info.filepath.name),
                    'slug': file_info.parameter.parameter.script_param if file_info.parameter else None}
        try:
            groups[file_info.filetype].append(filedict)
        except KeyError:
            groups[file_info.filetype] = [filedict]
        if file_info.filetype != 'all':
            groups['all'].append(filedict)
    return groups


from __future__ import absolute_import
import argparse
import sys
import json
import imp
import traceback
import tempfile
from .ast import source_parser
from itertools import chain

def is_upload(action):
    """Checks if this should be a user upload

    :param action:
    :return: True if this is a file we intend to upload from the user
    """
    # import pdb; pdb.set_trace();
    return 'r' in action.type._mode and (action.default is None or
                                         getattr(action.default, 'name') not in (sys.stderr.name, sys.stdout.name))

# input attributes we try to set:
# checked, name, type, value
# extra information we want to append:
# help,
# required,
# param (for executing the script and knowing if we need - or --),
# upload (boolean providing info on whether it's a file are we uploading or saving)
# choices (for selections)
# choice_limit (for multi selection)

CHOICE_LIMIT_MAP = {'?': '1', '+': '>=1', '*': '>=0'}

# We want to map to model fields as well as html input types we encounter in argparse
# keys are known variable types, as defined in __builtins__
# the model is a Django based model, which can be fitted in the future for other frameworks.
# The type is the HTML input type
# nullcheck is a function to determine if the default value should be checked  (for cases like default='' for strings)
# the attr_kwargs is a mapping of the action attributes to its related html input type. It is a dict
# of the form: {'name_for_html_input': {
# and either one or both of:
#  'action_name': 'attribute_name_on_action', 'callback': 'function_to_evaluate_action_and_return_value'} }

GLOBAL_ATTRS = ['model', 'type']

GLOBAL_ATTR_KWARGS = {
    'name': {'action_name': 'dest'},
    'value': {'action_name': 'default'},
    'required': {'action_name': 'required'},
    'help': {'action_name': 'help'},
    'param': {'callback': lambda x: x.option_strings[0]},
    'choices': {'callback': lambda x: x.choices},
    'choice_limit': {'callback': lambda x: CHOICE_LIMIT_MAP.get(x.nargs, x.nargs)}
    }

TYPE_FIELDS = {
    # Python Builtins
    bool: {'model': 'BooleanField', 'type': 'checkbox', 'nullcheck': lambda x: x.default is None,
           'attr_kwargs': GLOBAL_ATTR_KWARGS},
    file: {'model': 'FileField', 'type': 'file', 'nullcheck': lambda x: False,
           'attr_kwargs': GLOBAL_ATTR_KWARGS},
    float: {'model': 'FloatField', 'type': 'text', 'html5-type': 'number', 'nullcheck': lambda x: x.default is None,
            'attr_kwargs': GLOBAL_ATTR_KWARGS},
    int: {'model': 'IntegerField', 'type': 'text', 'nullcheck': lambda x: x.default is None,
          'attr_kwargs': GLOBAL_ATTR_KWARGS},
    None: {'model': 'CharField', 'type': 'text', 'nullcheck': lambda x: x.default is None,
          'attr_kwargs': GLOBAL_ATTR_KWARGS},
    str: {'model': 'CharField', 'type': 'text', 'nullcheck': lambda x: x.default == '' or x.default is None,
          'attr_kwargs': GLOBAL_ATTR_KWARGS},
    unicode: {'model': 'CharField', 'type': 'text', 'nullcheck': lambda x: x.default == '' or x.default is None,
              'attr_kwargs': GLOBAL_ATTR_KWARGS},

    # argparse Types
    argparse.FileType: {'model': 'FileField', 'type': 'file', 'nullcheck': lambda x: False,
                        'attr_kwargs': dict(GLOBAL_ATTR_KWARGS, **{
                            'value': None,
                            'required': {'callback': lambda x: x.required or x.default in (sys.stdout, sys.stdin)},
                            'upload': {'callback': is_upload}
                        })},
}

# There are cases where we can glean additional information about the form structure, e.g.
# a StoreAction with default=True can be different than a StoreTrueAction with default=False
ACTION_CLASS_TO_TYPE_FIELD = {
    argparse._StoreAction: dict(TYPE_FIELDS, **{

    }),
    argparse._StoreConstAction: dict(TYPE_FIELDS, **{

    }),
    argparse._StoreTrueAction: dict(TYPE_FIELDS, **{
        None: {'model': 'BooleanField', 'type': 'checkbox', 'nullcheck': lambda x: x.default is None,
               'attr_kwargs': dict(GLOBAL_ATTR_KWARGS, **{
                    'checked': {'callback': lambda x: x.default},
                    'value': None,
                    })
               },
    }),
    argparse._StoreFalseAction: dict(TYPE_FIELDS, **{
        None: {'model': 'BooleanField', 'type': 'checkbox', 'nullcheck': lambda x: x.default is None,
               'attr_kwargs': dict(GLOBAL_ATTR_KWARGS, **{
                    'checked': {'callback': lambda x: x.default},
                    'value': None,
                    })
               },
    })
}

class ArgParseNode(object):
    """
        This class takes an argument parser entry and assigns it to a Build spec
    """
    def __init__(self, action=None):
        fields = ACTION_CLASS_TO_TYPE_FIELD.get(type(action), TYPE_FIELDS)
        field_type = fields.get(action.type)
        if field_type is None:
            field_types = [i for i in fields.keys() if i is not None and issubclass(type(action.type), i)]
            if len(field_types) > 1:
                field_types = [i for i in fields.keys() if i is not None and isinstance(action.type, i)]
            if len(field_types) == 1:
                field_type = fields[field_types[0]]
        self.node_attrs = dict([(i, field_type[i]) for i in GLOBAL_ATTRS])
        null_check = field_type['nullcheck'](action)
        for attr, attr_dict in field_type['attr_kwargs'].iteritems():
            if attr_dict is None:
                continue
            if attr == 'value' and null_check:
                continue
            if 'action_name' in attr_dict:
                self.node_attrs[attr] = getattr(action, attr_dict['action_name'])
            elif 'callback' in attr_dict:
                self.node_attrs[attr] = attr_dict['callback'](action)

    @property
    def name(self):
        return self.node_attrs.get('name')

    def __str__(self):
        return json.dumps(self.node_attrs)

    def to_django(self):
        exclude = {'name', 'model'}
        field_module = 'models'
        django_kwargs = {}
        if self.node_attrs['model'] == 'CharField':
            django_kwargs['max_length'] = 255
        django_kwargs['blank'] = not self.node_attrs['required']
        try:
            django_kwargs['default'] = self.node_attrs['value']
        except KeyError:
            pass
        return u'{0} = {1}.{2}({3})'.format(self.node_attrs['name'], field_module, self.node_attrs['model'],
                                           ', '.join(['{0}={1}'.format(i,v) for i,v in django_kwargs.iteritems()]),)


class ArgParseNodeBuilder(object):
    def __init__(self, script_path=None, script_name=None):
        self.valid = True
        try:
            module = imp.load_source(script_name, script_path)
        except:
            sys.stderr.write('Error while loading {0}:\n'.format(script_path))
            self.error = '{0}\n'.format(traceback.format_exc())
            sys.stderr.write(self.error)
            self.valid = False
            return
        parsers = [v for i, v in chain(module.main.__globals__.iteritems(), vars(module).iteritems())
                   if issubclass(type(v), argparse.ArgumentParser)]
        if not parsers:
            f = tempfile.NamedTemporaryFile()
            ast_source = source_parser.parse_source_file(script_path)
            python_code = source_parser.convert_to_python(list(ast_source))
            f.write('\n'.join(python_code))
            f.seek(0)
            module = imp.load_source(script_name, f.name)
            parsers = [v for i, v in chain(module.main.__globals__.iteritems(), vars(module).iteritems())
                   if issubclass(type(v), argparse.ArgumentParser)]
        if not parsers:
            sys.stderr.write('Unable to identify ArgParser for {0}:\n'.format(script_path))
            self.valid = False
            return
        parser = parsers[0]
        self.class_name = script_name
        self.script_path = script_path
        self.script_description = getattr(parser, 'description', None)
        self.script_groups = []
        self.nodes = {}
        self.script_groups = []
        non_req = set([i.dest for i in parser._get_optional_actions()])
        self.optional_nodes = set([])
        self.containers = {}
        for action in parser._actions:
            # This is the help message of argparse
            if action.default == argparse.SUPPRESS:
                continue
            node = ArgParseNode(action=action)
            container = action.container.title
            container_node = self.containers.get(container, None)
            if container_node is None:
                container_node = []
                self.containers[container] = container_node
            self.nodes[node.name] = node
            container_node.append(node.name)
            if action.dest in non_req:
                self.optional_nodes.add(node.name)

    def get_script_description(self):
        return {'name': self.class_name, 'path': self.script_path,
                'description': self.script_description,
                'inputs': [{'group': container_name, 'nodes': [self.nodes[node].node_attrs for node in nodes]}
                           for container_name, nodes in self.containers.iteritems()]}

    @property
    def json(self):
        return json.dumps(self.get_script_description())


from __future__ import absolute_import
from django import forms
from ..backend import utils


class DjanguiForm(forms.Form):

    def add_djangui_fields(self):
        # This adds fields such as job name, description that we like to validate on but don't want to include in
        # form rendering
        self.fields['job_name'] = forms.CharField()
        self.fields['job_description'] = forms.CharField(required=False)

    def save(self, **kwargs):
        if 'user' in self.data:
            self.cleaned_data['user'] = self.data['user']
        job = utils.create_djangui_job(self.cleaned_data)
        return job


from __future__ import division, absolute_import
from django import template
from .. import settings as djangui_settings

register = template.Library()
@register.filter
def divide(value, arg):
    try:
        return float(value)/float(arg)
    except ZeroDivisionError:
        return None

@register.filter
def endswith(value, arg):
    return str(value).endswith(arg)

@register.filter
def valid_user(obj, user):
    from ..backend import utils
    valid = utils.valid_user(obj, user)
    return True if valid.get('valid') else valid.get('display')

@register.filter
def complete_job(status):
    from ..models import DjanguiJob
    from celery import states
    return status in (DjanguiJob.COMPLETED, states.REVOKED)


from __future__ import absolute_import

from django.db.models.signals import post_delete

from celery.signals import task_postrun, task_prerun, task_revoked


@task_postrun.connect
@task_prerun.connect
def task_completed(sender=None, **kwargs):
    task_kwargs = kwargs.get('kwargs')
    job_id = task_kwargs.get('djangui_job')
    from .models import DjanguiJob
    from celery import states
    job = DjanguiJob.objects.get(pk=job_id)
    state = kwargs.get('state')
    if state:
        job.status = DjanguiJob.COMPLETED if state == states.SUCCESS else state
    job.celery_id = kwargs.get('task_id')
    job.save()

def reload_scripts(**kwargs):
    from .backend import utils
    utils.load_scripts()

# TODO: Figure out why relative imports fail here
from djangui.models import Script, ScriptGroup, ScriptParameter, ScriptParameterGroup
post_delete.connect(reload_scripts, sender=Script)
post_delete.connect(reload_scripts, sender=ScriptGroup)
post_delete.connect(reload_scripts, sender=ScriptParameter)
post_delete.connect(reload_scripts, sender=ScriptParameterGroup)


__author__ = 'chris'
from django.conf import settings

def get(key, default):
    return getattr(settings, key, default)

DJANGUI_FILE_DIR = get('DJANGUI_FILE_DIR', 'djangui_files')
DJANGUI_SCRIPT_DIR = get('DJANGUI_SCRIPT_DIR', 'djangui_scripts')
DJANGUI_CELERY = get('DJANGUI_CELERY', True)
DJANGUI_CELERY_TASKS = get('DJANGUI_CELERY_TASKS', 'djangui.tasks')
DJANGUI_ALLOW_ANONYMOUS = get('DJANGUI_ALLOW_ANONYMOUS', True)
DJANGUI_AUTH = get('DJANGUI_AUTH', True)
DJANGUI_LOGIN_URL = get('DJANGUI_LOGIN_URL', settings.LOGIN_URL)
DJANGUI_REGISTER_URL = get('DJANGUI_REGISTER_URL', '/accounts/register/')
DJANGUI_SHOW_LOCKED_SCRIPTS = get('DJANGUI_SHOW_LOCKED_SCRIPTS', True)
DJANGUI_EPHEMERAL_FILES = get('DJANGUI_EPHEMERAL_FILES', False)


__author__ = 'chris'
from unittest import TestCase
import subprocess
import os
import shutil

BASE_DIR = os.path.split(__file__)[0]
DJANGUI_SCRIPT_PATH = os.path.join(BASE_DIR, '..', 'scripts', 'djanguify.py')
DJANGUI_TEST_PROJECT_NAME = 'djangui_project'
DJANGUI_TEST_PROJECT_PATH = os.path.join(BASE_DIR, DJANGUI_TEST_PROJECT_NAME)
DJANGUI_TEST_PROJECT_MANAGE = os.path.join(DJANGUI_TEST_PROJECT_PATH, 'manage.py')
PYTHON_INTERPRETTER = 'python'
DJANGUI_PROCESS = None
CELERY_PROCESS = None

class TestProject(TestCase):
    def setUp(self):
        # start the project
        # if old stuff exists, remove it
        if os.path.exists(DJANGUI_TEST_PROJECT_PATH):
            shutil.rmtree(DJANGUI_TEST_PROJECT_PATH)
        global DJANGUI_PROCESS
        global CELERY_PROCESS
        proc = subprocess.Popen([PYTHON_INTERPRETTER, DJANGUI_SCRIPT_PATH, '-p', DJANGUI_TEST_PROJECT_NAME], cwd=BASE_DIR)
        proc.wait()
        DJANGUI_PROCESS = subprocess.Popen([PYTHON_INTERPRETTER, DJANGUI_TEST_PROJECT_MANAGE, 'runserver'])
        CELERY_PROCESS = subprocess.Popen([PYTHON_INTERPRETTER, DJANGUI_TEST_PROJECT_MANAGE, 'celery', 'worker'])

    def tearDown(self):
        global DJANGUI_PROCESS
        global CELERY_PROCESS
        DJANGUI_PROCESS.kill()
        CELERY_PROCESS.kill()
        shutil.rmtree(DJANGUI_TEST_PROJECT_PATH)

    def test_simple(self):
        assert(True) == True



#!/usr/bin/env python

__author__ = 'chris'
import argparse
import sys

parser = argparse.ArgumentParser(description="https://projecteuler.net/problem=16 -- What is the sum of the digits of the number 2^1000?")
parser.add_argument('--exponent', help='The exponent to find digits below.', type=int, default=1)

def main():
    args = parser.parse_args()
    exponent = args.exponent
    if exponent  < 0:
        sys.stderr.write('Exponents must be positive.')
        return -1
    exponent = 5000 if exponent  > 1000 else exponent
    answer = sum(map(int, list(str(2**exponent))))
    sys.stdout.write('The sum of digits for 2^{} is {}'.format(exponent, answer))

if __name__ == "__main__":
    sys.exit(main())



import os
from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-djangui',
    version='0.1.7',
    packages=find_packages(),
    scripts=['scripts/djanguify.py'],
    install_requires = ['Django>=1.8', 'django-autoslug', 'django-celery'],
    include_package_data=True,
    license='GPLv3',
    description='A Django app which creates a web GUI and task interface for argparse scripts',
    url='http://www.github.com/chris7/djangui',
    author='Chris Mitchell',
    author_email='chris.mit7@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        # Replace these appropriately if you are stuck on Python 2.
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangui', '0002_auto_20150506_1728'),
    ]

    operations = [
        migrations.AddField(
            model_name='djanguijob',
            name='status',
            field=models.CharField(default=b'submitted', max_length=10, choices=[(b'submitted', 'Submitted'), (b'completed', 'Completed'), (b'deleted', 'Deleted')]),
        ),
        migrations.AlterField(
            model_name='scriptparameter',
            name='output_path',
            field=models.FilePathField(path=b'/home/chris/Devel/djtest/djtest/user_uploads', max_length=255, allow_files=False, recursive=True, allow_folders=True),
        ),
    ]



import factory
import os

from ..models import DjanguiJob, ScriptGroup, Script, ScriptParameter, ScriptParameterGroup, ScriptParameters

from . import config



class ScriptGroupFactory(factory.DjangoModelFactory):
    class Meta:
        model = ScriptGroup

    group_name = 'test group'
    group_description = 'test desc'

class ScriptFactory(factory.DjangoModelFactory):
    class Meta:
        model = Script

    script_name = 'test script'
    script_group = factory.SubFactory(ScriptGroupFactory)
    script_description = 'test script desc'

class TranslateScriptFactory(ScriptFactory):

    script_path = factory.django.FileField(from_path=os.path.join(config.DJANGUI_TEST_SCRIPTS, 'translate.py'))


import traceback
import sys

from django.apps import AppConfig

class DjanguiConfig(AppConfig):
    name = 'djangui'
    verbose_name = 'Djangui'

    def ready(self):
        from .backend import utils
        try:
            utils.load_scripts()
        except:
            sys.stderr.write('Unable to load scripts:\n{}\n'.format(traceback.format_exc()))
        from . import signals